package com.hand.hls.web.svc;

import com.hand.hap.core.web.view.IDGenerator;
import com.hand.hap.core.web.view.XMap;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-11-27
 * Time: 19:22
 */
public abstract class ServiceComponent extends ServiceTag {
    private static final String PROPERTY_ID = "id";
    public static final String PROPERTY_TEMPLATE = "template";
    private static final String PROPERTY_TEMPLATE_PREFIX = "tag/";
    private static final String PROPERTY_DEFAULT_TEMPLATE = "default";
    private XMap prototype;

    @Override
    public String execute(XMap view, ServiceContext context) throws Exception {
        StringBuilder sb = new StringBuilder();
        List<XMap> children = view.getChildren();
        if (children != null) {
            Iterator<XMap> it = children.iterator();
            while (it.hasNext()) {
                XMap child = it.next();
                sb.append(ServiceBuilder.build(child, context));
            }
        }
        context.put("content", sb.toString());
        return super.build(context, getTemplate(view));
    }

    protected void initPrototype(XMap obj) {
        this.prototype = obj;
    }

    public XMap getPrototype() {
        return prototype;
    }

    protected Object getProperty(String key) {
        return getPrototype().getPropertity(key);
    }

    protected void setProperty(Object key, Object value) {
        XMap prototype = getPrototype();
        if (prototype != null) {
            prototype.putPropertity(key, value);
        }
    }

    public String getId() {
        String id = getPrototype().getString(PROPERTY_ID);
        if (id == null) {
            id = "Y" + IDGenerator.getInstance().generate();
        }
        return id;
    }

    public void setId(String id) {
        setProperty(PROPERTY_ID, id);
    }

    @Override
    public void init(XMap view, ServiceContext context) throws Exception {
        initPrototype(view);

        context.put(PROPERTY_ID, getId());
        XMap prototype = getPrototype();
        Properties properties = MapUtils.toProperties(prototype);
        Enumeration<?> enumeration = properties.propertyNames();
        while (enumeration.hasMoreElements()) {
            Object key = enumeration.nextElement();
            Object value = properties.getOrDefault(key, null);
            context.put(String.valueOf(key), value);
        }
    }

    public String getTemplate() {
        String simpleName = this.getClass().getSimpleName();
        return PROPERTY_TEMPLATE_PREFIX + simpleName + ".ftl";
    }

    public String getTemplate(XMap view) {
        String tpl = view.getString(PROPERTY_TEMPLATE);
        String template = getTemplate();
        return StringUtils.isEmpty(tpl) ? StringUtils.isEmpty(template) ? PROPERTY_DEFAULT_TEMPLATE : template : tpl;
    }
}
