package com.hand.hls.web.svc.utils;

import com.hand.hls.web.svc.domain.Field;
import com.hand.hls.web.svc.domain.KeyField;
import com.hand.hls.web.svc.domain.Model;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-12-07
 * Time: 15:18
 */
public class ModelUpdateParser {
    public static String parseUpdate(Model model) {
        return parseUpdateWithParam(model, null);
    }

    public static String parseUpdateWithParam(Model model, Map<String, Object> param) {
        Assert.notNull(model, "invalid parameter");
        Assert.notNull(param, "invalid parameter");
        String tableName = model.getTableName();
        List<Field> fields = model.getFields();
        Assert.notNull(tableName, "invalid tableName");
        Assert.notEmpty(fields, "fields must not be empty");
        String where = parsePkFields(model, param);
        String setPart = StringUtils.join(fields.stream().filter(field -> param.containsKey(field.getParamPath())).map(field -> {
            String name = field.getName();
            return name + "=:" + field.getParamPath();
        }).collect(Collectors.toList()), " , ");
        if(StringUtils.isEmpty(setPart)){
            throw new IllegalArgumentException("non field will be updated");
        }
        return "update " + tableName + " set " + setPart + " where " + where;
    }

    private static String parsePkFields(Model model, Map<String, Object> param) {
        List<KeyField> primaryKey = model.getPrimaryKey();
        Assert.notEmpty(primaryKey, "update operation need at least one primary key be config");
        StringBuilder whereBuilder = new StringBuilder();
        for (int i = 0; i < primaryKey.size(); i++) {
            if (i > 0) {
                whereBuilder.append(" and ");
            }
            KeyField key = primaryKey.get(i);
            String name = key.getName();
            if (!param.containsKey(name)) {
                // pk not in params
                throw new IllegalArgumentException("each pk must have value in params");
            }
            whereBuilder.append(name).append("=:").append(key.getParamPath());
        }
        return whereBuilder.toString();
    }

    public static String parseUpdateWithParamOnly(Model model, Map allParams, Map passParams) {
        Assert.notNull(model, "invalid parameter");
        Assert.notNull(passParams, "invalid parameter");
        Assert.notNull(allParams, "invalid parameter");
        String tableName = model.getTableName();
        Assert.notNull(tableName, "invalid tableName");
        String wherePart = parsePkFields(model, allParams);
        List<String> sets = new ArrayList<>(passParams.size());
        passParams.forEach((k,v)->{
            String fieldName = String.valueOf(k);
            sets.add(" "+fieldName+"=:"+fieldName);
        });
        return "update "+tableName + " set " + StringUtils.join(sets, ",") + " where " + wherePart;
    }
}
